/***************************************************************************
 *
 *           Copyright 1998 by Carnegie Mellon University
 * 
 *                       All Rights Reserved
 * 
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose and without fee is hereby granted,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the name of CMU not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.
 * 
 * CMU DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
 * ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL
 * CMU BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR
 * ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
 * WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
 * ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
 * SOFTWARE.
 * 
 * Author: Ryan Troll <ryan+@andrew.cmu.edu>
 * 
 ***************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <snmp.h>

#include "version.h"

void Version (void)
{
  fprintf (stderr,
           "snmptranslate, version %s\n"
           "using CMU SNMP Library, version %s\n",
           snmpapps_Version(), snmp_Version());
  exit (1);
}

void Usage (void)
{
  fprintf (stderr, "usage: snmptran [-nvV] object-identifier\n");
  exit (-1);
}

int main (int argc, char **argv)
{
  char *current_name = NULL;
  oid   name[MAX_NAME_LEN];
  int   name_length = MAX_NAME_LEN;
  int   tosymbolic  = 0;
  int   ch, verbose = 0;

  /* Usage: snmptranslate [-ndvV] name
   */
  while ((ch = getopt(argc,argv, "?nvV")) != EOF)
        switch (ch)
        {
          case 'n':
               tosymbolic = 1;
               break;
          case 'v':
               verbose = 1;
               break;
          case 'V':
               Version();
               break;
          default:
               Usage();
        }

  argc -= optind;
  argv += optind;

  current_name = *argv++;
  if (current_name == NULL)
     Usage();

  init_mib();

  if (!read_objid(current_name, name, &name_length))
  {
    printf ("Invalid name '%s'!\n", current_name);
    return (-1);
  }

  if (verbose)
     printf ("Name: '%s'\n", current_name);

  if (tosymbolic)
  {
    if (verbose)
       printf ("Nums:  ");
    print_oid_nums (name, name_length);
    printf ("\n");
  }
  else
  {
    if (verbose)
       printf ("Words: ");
    print_objid (name, name_length);
  }
  return (0);
}
