/* 
 * Extra functions.  Just for fun.
 *
 * These functions were commenly used in SNMP Applications, so they were
 * put here.
 */

/**********************************************************************
 *
 *           Copyright 1998 by Carnegie Mellon University
 * 
 *                       All Rights Reserved
 * 
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose and without fee is hereby granted,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the name of CMU not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.
 * 
 * CMU DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
 * ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL
 * CMU BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR
 * ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
 * WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
 * ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
 * SOFTWARE.
 * 
 * Author: Ryan Troll <ryan+@andrew.cmu.edu>
 * 
 * Changes by G. Vanem 1998
 * Adapted to Waterloo tcp/ip
 **********************************************************************/

#include "snmp_lib.h"

#include <net/if.h>
#include <net/if_ether.h>
#include <net/if_dl.h>
#include <net/route.h>

/*
 * Turn timeticks into a string.  Returns a pointer to the string.
 */
char *uptime_string (DWORD timeticks, char *buf)
{
  int seconds, minutes, hours, days;

  timeticks /= 100;
  days = timeticks / (60 * 60 * 24);
  timeticks %= (60 * 60 * 24);

  hours = timeticks / (60 * 60);
  timeticks %= (60 * 60);

  minutes = timeticks / 60;
  seconds = timeticks % 60;

  switch (days)
  {
    case 0:  sprintf (buf, "%d:%02d:%02d", hours, minutes, seconds);
             break;
    case 1:  sprintf (buf, "%d day, %d:%02d:%02d", days,hours,minutes,seconds);
             break;
    default: sprintf (buf, "%d days, %d:%02d:%02d", days,hours,minutes,seconds);
             break;
  }
  return (buf);
}

DWORD myaddress (void)
{
  return htonl (my_ip_addr);
}

/*
 * Convert between various OID formats
 */
int mib_TxtToOid (char *Buf, oid ** OidP, int *LenP)
{
  char *bufp;
  oid  *NewOid;
  char *S;
  int   len   = 0;
  int   index = 0;
  int   val   = 0;

  /* Calculate initial length
   */
  for (bufp = Buf; *bufp; bufp++)
      if (*bufp == '.')
         len++;

  if (Buf[0] != '.')
     len++;

  *LenP = len;
  *OidP = malloc (sizeof(oid) * len);
  if (*OidP == NULL)
  {
    snmp_set_api_error (SNMPERR_OS_ERR);
    return (0);
  }
  NewOid = *OidP;

  /* Now insert bytes */
  S = Buf;
  if (*S == '.')
     ++S;

  MIBTT_DEBUG0 ("**********\n");
  MIBTT_DEBUG1 ("Txt: `%s'\n", Buf);
  MIBTT_DEBUG1 ("Len: (%d)\n", len);

  while (1)
  {
    val = atoi (S);
    while (!((*S == '\0') || (*S == '.')))
    {
      if (!isdigit ((int)*S))
      {
        snmp_set_api_error (SNMPERR_INVALID_TXTOID);
        return (0);
      }
      S++;
    }
    MIBTT_DEBUG2 ("%.2d: %d\n", index, val);

    /* End of a number.
     */
    NewOid[index++] = val;
    val = 0;

    /* End of string?
     */
    if (*S == '\0')
    {
      MIBTT_DEBUG0 ("Returning.\n");
      return (1);
    }

    /* Nope. On to the next.
     */
    S++;
  }
}

int mib_OidToTxt (oid *O, int OidLen, char *Buf, int BufLen)
{
  char *bufp;
  int   x, Len;
  int   MaxLen = BufLen-3;    /* Safety margin */

  bufp = Buf;
  Len  = 0;

  for (x = 0; x < OidLen; x++)
  {
    int len = sprintf (bufp, ".%lu", O[x]);
    Len  += len;
    bufp += len;
    if (Len > MaxLen)
    {
      snmp_set_api_error (SNMPERR_TOO_LONG);
      return (0);
    }
  }
  return (1);
}
