#include "snmp_lib.h"

#ifdef __BORLANDC__
#pragma warn -prot
#endif

DWORD snmpStats [SNMP_LAST_STAT+1] = { 0 };

void v2md5auth_password_to_key (
     char  *password,     /* IN */
     DWORD  passwordlen,  /* IN */
     BYTE  *agentID,      /* IN  - pointer to 12 octet long agentID */
     BYTE  *key)          /* OUT - caller supplies pointer to 16 octet buffer */
{
  MD5_CTX MD;
  BYTE    *cp, password_buf[64];
  DWORD   password_index = 0;
  DWORD   count = 0, i;

  MD5Init (&MD);   /* initialize MD5 */

  /* loop until we've done 1 Megabyte */
  while (count < 1048576)
  {
    cp = password_buf;
    for (i = 0; i < 64; i++)
    {
      *cp++ = password [password_index++ % passwordlen];
     /*
      * Take the next byte of the password, wrapping to the
      * beginning of the password as necessary.
      */
    }

    MD5Update (&MD, password_buf, 64);

    /*
     * 1048576 is divisible by 64, so the last MDupdate will be
     * aligned as well.
     */
    count += 64;
  }

  MD5Final (password_buf, &MD);
  memcpy (password_buf+16, agentID, 12);
  memcpy (password_buf+28, password_buf, 16);

  MD5Init (&MD);   /* initialize MD5 */
  MD5Update (&MD, password_buf, 44);
  MD5Final (key, &MD);
}

void md5Digest (BYTE *msg, int length, BYTE *key, BYTE *digest)
{
  MD5_CTX MD;

  MD5Init (&MD);   /* initialize MD5 */
  MD5Update (&MD, msg, length);
  MD5Update (&MD, key, 16);
  MD5Final (digest, &MD);
}

/*
 * this routine is used to parse a community string that is passed as
 * a command line option to an application (management entity) program
 */
int parse_app_community_string (struct snmp_session *session)
{
  char *cp = session->community;
  char *start;
  int  ch,i;

  /* community string is a v2u community if it begins with '/' */
  if (*cp == '/')
  {
    cp++;
    start = cp;
    while (*cp && *cp != '/')
           cp++;
    session->params.userLen = cp - start;
    strncpy (session->params.userName, start, cp - start);
    session->params.userName [cp - start] = 0;

    if (session->params.userLen == 0 )
    {
      (*_printf) ("userName cannot be zero length\n");
      return (-1);
    }

    session->params.qoS = 0;
    session->params.MMS = SNMP_MAX_LEN;
    session->Version    = SNMP_VERSION_2;

    /* look for authKey */
    if (*cp != '/')
       return (0);
    cp++;

    start = cp;
    while (*cp && *cp != '/')
          cp++;
    if ((cp - start > 2) && (strncmp(start,"0x",2) == 0))
    {
      if (cp - start != 34)
      {
        (*_printf) ("auth key not 16 octets\n");
        return (-1);
      }

      start += 2;
      for (i = 0; i < 16; i++)
      {
        if (sscanf(start,"%2x",&ch) != 1)
        {
          (*_printf) ("auth key contains non hex digits\n");
          return (-1);
        }
        start += 2;
        session->params.authKey[i] = ch;
      }
      session->params.qoS |= USEC_QOS_AUTH;
    }
    else if (cp - start > 0)
    {
      v2md5auth_password_to_key (start, cp - start,
                                 session->params.agentID,
                                 session->params.authKey);
      session->params.qoS |= USEC_QOS_AUTH;
    }

    /* look for privKey
     */
    if (*cp != '/')
       return (0);
    cp++;

    start = cp;
    while (*cp && *cp != '/')
          cp++;

    if ((cp - start > 2) && !strncmp (start,"0x",2))
    {
      if (cp - start != 34)
      {
        (*_printf) ("priv key not 16 octets\n");
        return (-1);
      }

      start += 2;
      for (i = 0; i < 16; i++)
      {
        if (sscanf(start, "%2x", &ch) != 1)
        {
          (*_printf) ("priv key contains non hex digits\n");
          return (-1);
        }
        session->params.privKey[i] = ch;
        start += 2;
      }
      session->params.qoS |= USEC_QOS_PRIV;
    }
    else if (cp - start > 0)
    {
      v2md5auth_password_to_key (start, cp - start,
                                 session->params.agentID,
                                 session->params.privKey);
      session->params.qoS |= USEC_QOS_PRIV;
    }

    /* look for contextSelector
     */
    if (*cp != '/')
       return (0);

    cp++;
    start = cp;
    while (*cp && *cp != '/')
          cp++;
    session->params.contextLen = cp - start;
    strncpy ((char*)session->params.contextSelector, start, cp - start);
  }
  return (0);
}

void snmp_increment_stat (int stat)
{
  snmpStats [stat]++;
}

void create_report (struct snmp_session *session,
                    BYTE                *out_data,
                    int                 *out_length,
                    int                  stat,
                    int                  reqid)
{
  struct snmp_pdu *report;
  static oid name[11] = { 1,3,6,1,6,3,0,1,1,0,0 };
  int    name_length  = 11;

  switch (stat)
  {
    case SNMP_STAT_ENCODING_ERRORS:
         name[6] = 1;
         name[8] = 1;
         name[9] = 3;
         break;
    case USEC_STAT_UNSUPPORTED_QOS:
         name[6] = 6;
         name[8] = 2;
         name[9] = 1;
         break;
    case USEC_STAT_NOT_IN_WINDOWS:
         name[6] = 6;
         name[8] = 2;
         name[9] = 2;
         break;
    case USEC_STAT_UNKNOWN_USERNAMES:
         name[6] = 6;
         name[8] = 2;
         name[9] = 3;
         break;
    case USEC_STAT_WRONG_DIGEST_VALUES:
         name[6] = 6;
         name[8] = 2;
         name[9] = 4;
         break;
    case USEC_STAT_UNKNOWN_CONTEXT_SELECTORS:
         name[6] = 6;
         name[8] = 2;
         name[9] = 5;
         break;
    case SNMP_STAT_BAD_OPERATIONS:
         name[6] = 1;
         name[8] = 1;
         name[9] = 11;
         break;
    case SNMP_STAT_PROXY_DROPS:
         name[6] = 1;
         name[8] = 1;
         name[9] = 13;
         break;
    case SNMP_STAT_SILENT_DROPS:
         name[6] = 1;
         name[8] = 1;
         name[9] = 12;
         break;
    default:
         return;
  }

  if ((session->params.qoS & USEC_QOS_AUTH) && (stat == USEC_STAT_NOT_IN_WINDOWS))
       session->params.qoS = USEC_QOS_AUTH;
  else session->params.qoS = USEC_QOS_NOAUTH_NOPRIV;

  report = snmp_pdu_create (SNMP_PDU_REPORT);
  report->errstat  = 0;
  report->errindex = 0;
  report->reqid    = reqid;
  snmp_add_null_var (report, name, name_length);
  report->variables->type       = TYPE_COUNTER;
  report->variables->val.string = malloc (sizeof(DWORD));
  report->variables->val_len    = sizeof(DWORD);
  *(DWORD*) report->variables->val.string = snmpStats [stat];
  snmp_build (session, report, out_data, out_length);

  snmp_free_pdu (report);
}

int parse_parameters (BYTE *pp, int plen, struct Parameters *params)
{
  /* 25 octets -- <model><qoS><agentID><agentBoots><agentTime><mms><userLen>
   */
  if (plen < 25)
     return (USEC_STAT_BAD_PARAMETERS);

  params->securityModel = pp[0];
  if (params->securityModel != SNMP_USEC_MODEL)
     return (USEC_STAT_BAD_PARAMETERS);

  params->qoS = pp[1];
  if ((params->qoS & ~(USEC_QOS_AUTH|USEC_QOS_GENREPORT)))
     return (USEC_STAT_BAD_PARAMETERS);

  memcpy (params->agentID, &pp[2], sizeof(params->agentID));

  params->agentBoots = (pp[14] << 24) + (pp[15] << 16) + (pp[16] << 8) + pp[17];
  params->agentTime  = (pp[18] << 24) + (pp[19] << 16) + (pp[20] << 8) + pp[21];
  params->MMS        = (pp[22] << 8) + pp[23];
  if (params->MMS < 484)
     return (USEC_STAT_BAD_PARAMETERS);

  params->userLen = pp[24];
  if (params->userLen == 0 || params->userLen > sizeof(params->userName))
     return (USEC_STAT_BAD_PARAMETERS);

  plen = plen - 25 - params->userLen;
  pp += 25;
  if (plen < 1)
     return (USEC_STAT_BAD_PARAMETERS);

  memcpy (params->userName, pp, params->userLen);
  pp += params->userLen;

  params->authLen = *pp++;
  plen--;
  if (params->qoS & USEC_QOS_AUTH)
  {
    if (plen < 16 || params->authLen != 16)
       return (USEC_STAT_BAD_PARAMETERS);
    memcpy (params->authDigest, pp, 16);
    params->authDigestPtr = pp;
    pp   += 16;
    plen -= 16;
  }
  else if (params->authLen != 0)
          return (USEC_STAT_BAD_PARAMETERS);

  params->contextLen = plen;
  if (params->contextLen > sizeof(params->contextSelector))
     return (USEC_STAT_BAD_PARAMETERS);

  memcpy (params->contextSelector, pp, params->contextLen);
  return (0);
}

int check_received_pkt (BYTE *pkt, int pktlen, BYTE *comm, int commlen,
                        struct snmp_session *session, struct snmp_pdu *pdu)
{
  struct Parameters *params;

  snmp_increment_stat (SNMP_STAT_PACKETS);

  params = &pdu->params;
  if (parse_parameters (comm,commlen,params))
     return (USEC_STAT_BAD_PARAMETERS);

  if (memcmp(session->params.agentID, params->agentID, sizeof(params->agentID)))
     return (USEC_STAT_UNKNOWN_CONTEXT_SELECTORS);

  if (session->params.contextLen != params->contextLen ||
      memcmp (session->params.contextSelector,
              params->contextSelector, session->params.contextLen))
     return (USEC_STAT_UNKNOWN_CONTEXT_SELECTORS);

  if (session->params.userLen != params->userLen ||
      memcmp (session->params.userName, params->userName,
              session->params.userLen))
     return (USEC_STAT_UNKNOWN_USERNAMES);

  /*
   * qoS must be of acceptable level for the userName
   * reports do not have to match the level requested
   */
  if (pdu->command != SNMP_PDU_REPORT &&
      (params->qoS & USEC_QOS_AUTHPRIV) <
       (session->params.qoS & USEC_QOS_AUTHPRIV))
     return (USEC_STAT_UNSUPPORTED_QOS);

  if (params->qoS & USEC_QOS_AUTH)
  {
    memcpy (params->authDigestPtr, session->params.authKey,
            sizeof(params->authKey));
    md5Digest (pkt, pktlen, session->params.authKey, params->authDigestPtr);
    if (memcmp(params->authDigest, params->authDigestPtr, sizeof(params->authKey)))
       return (USEC_STAT_WRONG_DIGEST_VALUES);

    if (params->agentBoots < session->params.agentBoots)
       return (USEC_STAT_NOT_IN_WINDOWS);

    if (params->agentBoots == session->params.agentBoots)
    {
      int lower = session->params.agentClock + time(NULL) -
                  SNMP_MESSAGE_LIFETIME;
      if (lower < 0)
          lower = 0;
      if (params->agentTime < lower)
         return (USEC_STAT_NOT_IN_WINDOWS);
    }

    if (params->agentBoots > session->params.agentBoots ||
        params->agentTime  > session->params.agentTime)
    {
      /* update the LCD */
      session->params.agentBoots = params->agentBoots;
      session->params.agentTime  = params->agentTime;
      session->params.agentClock = params->agentTime - time(NULL);
    }
  }
  return (0);
}
